<?php

class FrmGetResponseAPI {

	protected $url = 'https://api.getresponse.com/v3';
	protected $api_key;
	protected $debug = 0;

	public function __construct() {
		$settings      = new FrmGetResponseSettings();
		$this->api_key = $settings->settings->api_key;
		$this->debug   = $settings->settings->debug_mode;
	}

	/**
	 * Get Campaigns from GetResponse
	 *
	 * @author Aman Saini
	 * @since  1.0
	 * @return Campaigns Array
	 */
	public function fetch_campaigns() {
		$lists = $this->remote_request( '/campaigns?fields=campaignId,name', array( 'method' => 'GET' ) );
		if ( ! is_array( $lists ) ) {
			$lists = false;
		}
		return $lists;
	}

	/**
	 * Get User defined Custom fields from GetResponse
	 *
	 * @author Aman Saini
	 * @since  1.0
	 * @return Custom Fields Array
	 */
	public function fetch_custom_fields() {
		return $this->remote_request( '/custom-fields?fields=customFieldId,name', array( 'method' => 'GET' ) );
	}

	/**
	 * Add user to getResponse
	 *
	 * @author Aman Saini
	 * @since  1.0
	 * @return Status Code
	 */
	public function subscribe_to_campaign( $subscriber ) {
		$body = json_encode( $subscriber );
		return $this->remote_request( '/contacts', compact( 'body' ) );
	}

	private function remote_request( $endpoint, $args = array() ) {
		$request  = $this->prepare_request( $args );
		$response = wp_remote_retrieve_body(
			wp_remote_request( $this->url . $endpoint, $request )
		);

		$this->print_debug( $response );

		if ( is_wp_error( $response ) ) {
			$response = false;
		}

		if ( $response ) {
			$response = json_decode( $response );
		}

		return $response;
	}

	private function prepare_request( $args ) {
		$request = array(
			'method'  => isset( $args['method'] ) ? $args['method'] : 'POST',
			'headers' => array(
				'content-type' => 'application/json',
				'X-Auth-Token' => 'api-key ' . $this->api_key,
			),
		);
		if ( isset( $args['body'] ) ) {
			$request['body'] = $args['body'];
		}

		return $request;
	}

	private function print_debug( $response ) {
		$show_message = strpos( $response, '"code":' ) || ! FrmAppHelper::doing_ajax();
		if ( ! empty( $this->debug ) && ! empty( $response ) && $show_message ) {
			echo '<pre>';
			print_r( $response );
			echo '</pre>';
		}
	}
}
